@@ -1,46 +1,49 @@
 # @package      hubzero-submit-distributor
 # @file         SitesInfo.py
 # @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012-2014 HUBzero Foundation, LLC.
-# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
+# @copyright    Copyright (c) 2012-2015 HUBzero Foundation, LLC.
+# @license      http://opensource.org/licenses/MIT MIT
 #
-# Copyright (c) 2012-2014 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2015 HUBzero Foundation, LLC.
 #
-# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
-# software: you can redistribute it and/or modify it under the terms of
-# the GNU Lesser General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any
-# later version.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
-# HUBzero is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
 #
 # HUBzero is a registered trademark of HUBzero Foundation, LLC.
 #
-
 import os
 import re
 import random
 import copy
 import socket
 import logging
+from xml.dom import minidom, Node
 
 from hubzero.submit.LogMessage      import getLogJobIdMessage as getLogMessage
 from hubzero.submit.GroupMembership import isGroupMember
 
 class SitesInfo:
    def __init__(self,
-                infoDirectory,
-                sitesFile,
-                allowedVenueMechanisms,
-                pegasusVersion):
+                sitesPath,
+                restrictionUser=None,
+                templateDirectory=None,
+                allowedVenueMechanisms=[],
+                pegasusVersion=None):
       self.logger = logging.getLogger(__name__)
       self.sites  = {}
 
@@ -49,97 +52,128 @@ class SitesInfo:
       commentPattern  = re.compile('\s*#.*')
       siteName        = ""
 
-      sitesPath = os.path.join(infoDirectory,sitesFile)
       if os.path.exists(sitesPath):
-         fpInfo = open(sitesPath,'r')
-         if fpInfo:
-            eof = False
-            while not eof:
-               record = fpInfo.readline()
-               if record != "":
-                  record = commentPattern.sub("",record)
-                  if   sitePattern.match(record):
-                     siteName = sitePattern.match(record).group(2)
-                     self.sites[siteName] = {'venues':[],
-                                             'gsiHost':socket.gethostname(),
-                                             'cloudDesignator':'',
-                                             'tunnelDesignator':'',
-                                             'siteMonitorDesignator':'',
-                                             'venueMechanism':'',
-                                             'venueGatekeeper':',,'.split(','),
-                                             'remoteUser':'',
-                                             'remoteBatchAccount':'',
-                                             'remoteBatchSystem':'',
-                                             'remoteBatchQueue':'',
-                                             'remoteBatchPartition':'',
-                                             'remoteBatchPartitionSize':'',
-                                             'remoteBatchConstraints':'',
-                                             'parallelEnvironment':'',
-                                             'remoteBinDirectory':os.path.join('${HOME}','bin'),
-                                             'remoteApplicationRootDirectory':'',
-                                             'remoteScratchDirectory':'',
-                                             'remotePpn':'',
-                                             'remoteManager':'',
-                                             'remoteHostAttribute':'',
-                                             'stageFiles':True,
-                                             'passUseEnvironment':False,
-                                             'arbitraryExecutableAllowed':True,
-                                             'executableClassificationsAllowed':['*'],
-                                             'members':[],
-                                             'state':'enabled',
-                                             'failoverSite':'None',
-                                             'checkProbeResult':True,
-                                             'restrictedToUsers':[],
-                                             'restrictedToGroups':[],
-                                             'logUserRemotely':False,
-                                             'undeclaredSiteSelectionWeight':0.,
-                                             'minimumWallTime':0.,
-                                             'maximumWallTime':0.,
-                                             'minimumCores':1,
-                                             'maximumCores':0,
-                                             'pegasusTemplates':[],
-                                             'pegasusSites':[]
-                                            }
-                  elif keyValuePattern.match(record):
-                     key,value = keyValuePattern.match(record).group(2,4)
-                     if key in self.sites[siteName]:
-                        if   isinstance(self.sites[siteName][key],list):
-                           self.sites[siteName][key] = [e.strip() for e in value.split(',')]
-                        elif isinstance(self.sites[siteName][key],bool):
-                           self.sites[siteName][key] = bool(value.lower() == 'true')
-                        elif isinstance(self.sites[siteName][key],float):
-                           self.sites[siteName][key] = float(value)
-                        elif isinstance(self.sites[siteName][key],int):
-                           self.sites[siteName][key] = int(value)
+         try:
+            fpInfo = open(sitesPath,'r')
+            try:
+               eof = False
+               while not eof:
+                  record = fpInfo.readline()
+                  if record != "":
+                     record = commentPattern.sub("",record)
+                     if   sitePattern.match(record):
+                        siteName = sitePattern.match(record).group(2)
+                        self.sites[siteName] = {'venues':[],
+                                                'venuePort':22,
+                                                'gsiHost':socket.gethostname(),
+                                                'cloudDesignator':'',
+                                                'tunnelDesignator':'',
+                                                'siteMonitorDesignator':'',
+                                                'identityManagers':[],
+                                                'venueMechanism':'',
+                                                'venueGatekeeper':',,'.split(','),
+                                                'remoteUser':'',
+                                                'remoteBatchAccount':'',
+                                                'remoteBatchSystem':'',
+                                                'remoteBatchQueue':'',
+                                                'remoteBatchPartition':'',
+                                                'remoteBatchPartitionSize':'',
+                                                'remoteBatchConstraints':'',
+                                                'parallelEnvironment':'',
+                                                'remoteBinDirectory':os.path.join('${HOME}','bin'),
+                                                'remoteApplicationRootDirectory':'',
+                                                'remoteScratchDirectory':'',
+                                                'remotePpn':'',
+                                                'remoteManager':'',
+                                                'remoteHostAttribute':'',
+                                                'stageFiles':True,
+                                                'passUseEnvironment':False,
+                                                'arbitraryExecutableAllowed':True,
+                                                'executableClassificationsAllowed':['*'],
+                                                'members':[],
+                                                'state':'enabled',
+                                                'failoverSite':'None',
+                                                'checkProbeResult':True,
+                                                'restrictedToUsers':[],
+                                                'restrictedToGroups':[],
+                                                'logUserRemotely':False,
+                                                'undeclaredSiteSelectionWeight':0.,
+                                                'minimumWallTime':0.,
+                                                'maximumWallTime':0.,
+                                                'minimumCores':1,
+                                                'maximumCores':0,
+                                                'minimumCoreWallTime':0.,
+                                                'maximumCoreWallTime':0.,
+                                                'pegasusTemplates':[],
+                                                'pegasusSites':[]
+                                               }
+                     elif keyValuePattern.match(record):
+                        key,value = keyValuePattern.match(record).group(2,4)
+                        if key in self.sites[siteName]:
+                           if   isinstance(self.sites[siteName][key],list):
+                              self.sites[siteName][key] = [e.strip() for e in value.split(',')]
+                           elif isinstance(self.sites[siteName][key],bool):
+                              self.sites[siteName][key] = bool(value.lower() == 'true')
+                           elif isinstance(self.sites[siteName][key],float):
+                              self.sites[siteName][key] = float(value)
+                           elif isinstance(self.sites[siteName][key],int):
+                              self.sites[siteName][key] = int(value)
+                           elif isinstance(self.sites[siteName][key],dict):
+                              try:
+                                 sampleKey   = self.sites[siteName][key].keys()[0]
+                                 sampleValue = self.sites[siteName][key][sampleKey]
+                              except:
+                                 sampleKey   = "key"
+                                 sampleValue = "value"
+                              self.sites[siteName][key] = {}
+                              for e in value.split(','):
+                                 dictKey,dictValue = e.split(':')
+                                 if isinstance(sampleKey,int):
+                                    dictKey = int(dictKey)
+                                 if   isinstance(sampleValue,int):
+                                    dictValue = int(dictValue)
+                                 elif isinstance(sampleValue,float):
+                                    dictValue = float(dictValue)
+                                 elif isinstance(sampleValue,bool):
+                                    dictValue = bool(dictValue.lower() == 'true')
+                                 self.sites[siteName][key][dictKey] = dictValue
+                           else:
+                              self.sites[siteName][key] = value
                         else:
-                           self.sites[siteName][key] = value
-                     else:
-                        message = "Undefined key = value pair %s = %s for site %s" % (key,value,siteName)
-                        self.logger.log(logging.WARNING,getLogMessage(message))
-               else:
-                  eof = True
-            fpInfo.close()
+                           message = "Undefined key = value pair %s = %s for site %s" % (key,value,siteName)
+                           self.logger.log(logging.WARNING,getLogMessage(message))
+                  else:
+                     eof = True
+            except (IOError,OSError):
+               self.logger.log(logging.ERROR,getLogMessage("Sites configuration file %s could not be read" % (sitesPath)))
+            finally:
+               fpInfo.close()
+         except (IOError,OSError):
+            self.logger.log(logging.ERROR,getLogMessage("Sites configuration file %s could not be opened" % (sitesPath)))
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("Sites configuration file %s is missing" % (sitesPath)))
 
-         for site in self.sites:
-            arbitraryExecutableAllowed = self.sites[site]['arbitraryExecutableAllowed']
-            executableClassificationsAllowed = self.sites[site]['executableClassificationsAllowed']
-            if len(executableClassificationsAllowed) == 1:
-               if executableClassificationsAllowed[0] == '*':
-                  if not arbitraryExecutableAllowed:
-                     self.sites[site]['executableClassificationsAllowed'] = ['staged', 'apps']
-            del self.sites[site]['arbitraryExecutableAllowed']
+      for site in self.sites:
+         arbitraryExecutableAllowed = self.sites[site]['arbitraryExecutableAllowed']
+         executableClassificationsAllowed = self.sites[site]['executableClassificationsAllowed']
+         if len(executableClassificationsAllowed) == 1:
+            if executableClassificationsAllowed[0] == '*':
+               if not arbitraryExecutableAllowed:
+                  self.sites[site]['executableClassificationsAllowed'] = ['staged', 'apps']
+         del self.sites[site]['arbitraryExecutableAllowed']
 
+      if not '*' in allowedVenueMechanisms:
          for site in self.sites:
             venueMechanism = self.sites[site]['venueMechanism']
             if venueMechanism != '':
                if not venueMechanism in allowedVenueMechanisms:
                   self.sites[site]['state'] = 'restrictedByVenueMechanism'
 
-         userName = os.getenv("USER")
+      if restrictionUser:
          for site in self.sites:
             restrictedToUsers = self.sites[site]['restrictedToUsers']
             if len(restrictedToUsers) > 0:
-               if not userName in restrictedToUsers:
+               if not restrictionUser in restrictedToUsers:
                   self.sites[site]['state'] = 'restrictedByUser'
 
          for site in self.sites:
@@ -154,50 +188,69 @@ class SitesInfo:
                   if not groupOK:
                      self.sites[site]['state'] = 'restrictedByGroup'
 
+      if not pegasusVersion:
          for site in self.sites:
             if self.sites[site]['state'] == 'enabled':
                if self.sites[site]['remoteBatchSystem'] == 'PEGASUS':
-                  if not pegasusVersion:
-                     self.sites[site]['state'] = 'restrictedByPegasus'
+                  self.sites[site]['state'] = 'restrictedByPegasus'
 
-         for site in self.sites:
-            members = self.sites[site]['members']
-            if len(members) > 0:
-               markedForDeletion = []
-               for member in members:
-                  if member in self.sites:
-                     if self.sites[member]['state'] != 'enabled':
-                        markedForDeletion.append(member)
-               for member in markedForDeletion:
-                  self.sites[site]['members'].remove(member)
-               del markedForDeletion
+      for site in self.sites:
+         members = self.sites[site]['members']
+         if len(members) > 0:
+            markedForDeletion = []
+            for member in members:
+               if member in self.sites:
+                  if self.sites[member]['state'] != 'enabled':
+                     markedForDeletion.append(member)
+            for member in markedForDeletion:
+               self.sites[site]['members'].remove(member)
+            del markedForDeletion
 
-         for site in self.sites:
-            failoverSite = self.sites[site]['failoverSite']
-            if failoverSite != 'None' and failoverSite != 'grid':
-               if self.sites[failoverSite]['state'] != 'enabled':
-                  self.sites[site]['failoverSite'] = 'None'
+      for site in self.sites:
+         failoverSite = self.sites[site]['failoverSite']
+         if failoverSite != 'None' and failoverSite != 'grid':
+            if self.sites[failoverSite]['state'] != 'enabled':
+               self.sites[site]['failoverSite'] = 'None'
 
-         for site in self.sites:
-            pegasusTemplates = copy.copy(self.sites[site]['pegasusTemplates'])
-            self.sites[site]['pegasusTemplates'] = {}
-            self.sites[site]['pegasusTemplates']['rc']    = ""
-            self.sites[site]['pegasusTemplates']['sites'] = ""
-            self.sites[site]['pegasusTemplates']['tc']    = ""
+      for site in self.sites:
+         pegasusTemplates = copy.copy(self.sites[site]['pegasusTemplates'])
+         self.sites[site]['pegasusTemplates'] = {}
+         self.sites[site]['pegasusTemplates']['rc']    = ""
+         self.sites[site]['pegasusTemplates']['sites'] = ""
+         self.sites[site]['pegasusTemplates']['tc']    = ""
+         if templateDirectory:
             for pegasusTemplate in pegasusTemplates:
-               pegasusTemplatePath = os.path.join(infoDirectory,pegasusTemplate)
+               pegasusTemplatePath = os.path.join(templateDirectory,pegasusTemplate)
                if os.path.exists(pegasusTemplatePath):
                   if   pegasusTemplate.endswith('.rc'):
                      self.sites[site]['pegasusTemplates']['rc']    = pegasusTemplatePath
                   elif pegasusTemplate.endswith('.xml'):
                      self.sites[site]['pegasusTemplates']['sites'] = pegasusTemplatePath
+                     pegasusSites = []
+                     siteDoc = minidom.parse(pegasusTemplatePath)
+                     rootNode = siteDoc.documentElement
+                     children = rootNode.getElementsByTagName("site")
+                     for child in children:
+                        handle = child.getAttribute('handle')
+                        if handle != 'local':
+                           gridstartDistribute = False
+                           for grandChild in child.getElementsByTagName("profile"):
+                              if grandChild.nodeType == Node.ELEMENT_NODE:
+                                 if grandChild.getAttribute("namespace") == "pegasus" and \
+                                    grandChild.getAttribute("key") == "gridstart":
+                                    if grandChild.hasChildNodes():
+                                       for greatGrandChild in grandChild.childNodes:
+                                          gridstart = greatGrandChild.nodeValue
+                                          if gridstart == 'Distribute':
+                                             gridstartDistribute = True
+                           pegasusSites.append((gridstartDistribute,handle))
+                     pegasusSites.sort()
+                     self.sites[site]['pegasusSites'] = [ handle for gridstartDistribute,handle in pegasusSites ]
                   elif pegasusTemplate.endswith('.txt'):
                      self.sites[site]['pegasusTemplates']['tc']    = pegasusTemplatePath
                else:
                   self.logger.log(logging.ERROR,getLogMessage("Pegasus template file %s does not exist." % (pegasusTemplatePath)))
             del pegasusTemplates
-      else:
-         self.logger.log(logging.ERROR,getLogMessage("Sites configuration file %s is missing" % (sitesPath)))
 
 
    def getEnabledSites(self):
@@ -233,9 +286,13 @@ class SitesInfo:
                           siteName):
       try:
          site = self.sites[siteName]
-         checkProbeResult = site['checkProbeResult']
       except:
          checkProbeResult = True
+      else:
+         if site['state'] == 'enabled':
+            checkProbeResult = site['checkProbeResult']
+         else:
+            checkProbeResult = True
 
       return(checkProbeResult)
 
@@ -310,6 +367,13 @@ class SitesInfo:
                if float(jobWallTime) > site['maximumWallTime']:
                   markedForDeletion.append(siteName)
 
+            jobCoreWallTime = nCores * float(jobWallTime)
+            if jobCoreWallTime < site['minimumCoreWallTime']:
+               markedForDeletion.append(siteName)
+            if site['maximumCoreWallTime'] > 0.:
+               if jobCoreWallTime > site['maximumCoreWallTime']:
+                  markedForDeletion.append(siteName)
+
             members = site['members']
             if len(members) > 0:
                membersMarkedForDeletion = []
@@ -332,7 +396,8 @@ class SitesInfo:
                            if float(jobWallTime) > memberSite['maximumWallTime']:
                               membersMarkedForDeletion.append(member)
                for member in membersMarkedForDeletion:
-                  site['members'].remove(member)
+                  if member in site['members']:
+                     site['members'].remove(member)
                del membersMarkedForDeletion
                if len(site['members']) == 0:
                   markedForDeletion.append(siteName)
@@ -608,6 +673,18 @@ class SitesInfo:
       return(selectedPegasusSite)
 
 
+   def getSitePegasusSites(self,
+                           siteName):
+      pegasusSites = ""
+      if siteName in self.sites:
+         site = self.sites[siteName]
+         if 'pegasusSites' in site:
+            if len(site['pegasusSites']) > 0:
+               pegasusSites = ','.join(site['pegasusSites'])
+
+      return(pegasusSites)
+
+
    def getSitesWithKeyValue(self,
                             key,
                             value,
@@ -688,8 +765,8 @@ class SitesInfo:
 
    def getExpandedSiteNames(self,
                             siteNames,
-                            remoteProbeMonitor):
-      expandedSiteNames = []
+                            remoteProbeMonitor=None):
+      expandedSiteNames = set()
 
       for siteName in siteNames:
          if siteName in self.sites:
@@ -700,20 +777,21 @@ class SitesInfo:
                   if member in self.sites:
                      if remoteProbeMonitor and self.__checkProbeResult(member):
                         if remoteProbeMonitor.isSiteAvailable(member):
-                           expandedSiteNames.append(member)
+                           expandedSiteNames.add(member)
                      else:
-                        expandedSiteNames.append(member)
+                        expandedSiteNames.add(member)
                   else:
-                     expandedSiteNames.append(siteName)
+                     expandedSiteNames.add(siteName)
             else:
                if remoteProbeMonitor and self.__checkProbeResult(siteName):
                   if remoteProbeMonitor.isSiteAvailable(siteName):
-                     expandedSiteNames.append(siteName)
+                     expandedSiteNames.add(siteName)
                else:
-                  expandedSiteNames.append(siteName)
+                  expandedSiteNames.add(siteName)
          else:
-            expandedSiteNames.append(siteName)
+            expandedSiteNames.add(siteName)
 
+      expandedSiteNames = list(expandedSiteNames)
       random.shuffle(expandedSiteNames)
 
       return(expandedSiteNames)
@@ -721,13 +799,16 @@ class SitesInfo:
 
    def getDefaultSiteInfo(self):
       siteInfo = {}
+      siteInfo['siteName']                         = "default"
       siteInfo['venues']                           = []
       siteInfo['venueIndex']                       = -1
       siteInfo['venue']                            = "Unknown"
+      siteInfo['venuePort']                        = 22
       siteInfo['venueMechanism']                   = ''
       siteInfo['cloudDesignator']                  = ""
       siteInfo['tunnelDesignator']                 = ""
       siteInfo['siteMonitorDesignator']            = ""
+      siteInfo['identityManagers']                 = []
       siteInfo['venueGatekeeper']                  = ""
       siteInfo['remoteUser']                       = ""
       siteInfo['sharedUserSpace']                  = False
@@ -758,6 +839,7 @@ class SitesInfo:
       siteInfo = {}
 
       site = self.sites[siteName]
+      siteInfo['siteName']                         = siteName
       siteInfo['venues']                           = site['venues']
       if len(siteInfo['venues']) > 0:
          siteInfo['venueIndex']                    = random.randint(0,len(siteInfo['venues'])-1)
@@ -765,12 +847,14 @@ class SitesInfo:
       else:
          siteInfo['venueIndex']                    = -1
          siteInfo['venue']                         = ""
+      siteInfo['venuePort']                        = site['venuePort']
       siteInfo['gsiHost']                          = site['gsiHost']
- 
+
       siteInfo['venueMechanism']                   = site['venueMechanism']
       siteInfo['cloudDesignator']                  = site['cloudDesignator']
       siteInfo['tunnelDesignator']                 = site['tunnelDesignator']
       siteInfo['siteMonitorDesignator']            = site['siteMonitorDesignator']
+      siteInfo['identityManagers']                 = site['identityManagers']
       siteInfo['venueGatekeeper']                  = site['venueGatekeeper']
       siteInfo['remoteUser']                       = site['remoteUser']
       if   siteInfo['remoteUser'].startswith('USER:'):
